/*
* NOTAS:
     - Put the pull-down resistors on the PORTD
     - Use 5V-3.3V voltage translator
     - Used pins of the Telit GM862 module:
                                              1,3,5,7 VBATT
                                              2,4,8  GND
                                              17  ON/OFF
                                              20  TXD   -  MCU's RC6
                                              37  RXD   -  MCU's RC7
                                              45  RTS   -  MCU's RC2
*/
// CONFIGURACION LCD
sbit LCD_RS at RB5_bit;
sbit LCD_EN at RB4_bit;
sbit LCD_D4 at RB3_bit;
sbit LCD_D5 at RB2_bit;
sbit LCD_D6 at RB1_bit;
sbit LCD_D7 at RB0_bit;

sbit LCD_RS_Direction at TRISB5_bit;
sbit LCD_EN_Direction at TRISB4_bit;
sbit LCD_D4_Direction at TRISB3_bit;
sbit LCD_D5_Direction at TRISB2_bit;
sbit LCD_D6_Direction at TRISB1_bit;
sbit LCD_D7_Direction at TRISB0_bit;
// FIN LCD

//CONFIGURACION DE RX
sbit RX_Flag at PIR1.B5;

// COMANDOS AT
const char atc1[] = "ATE0";                      // disable echo
const char atc2[] = "AT+CFUN=1";                 // set full functionality
const char atc3[] = "ATD9611029611;";             // place a call to phone number 123456789
                                                 // instead of 123456789 insert your phone number
const char atc4[] = "ATH";                       // hang up
const char atc5[] = "ATA";                       // answer a call
//const char atc_bnd[] = "AT#BND=1";             // GSM900+PCS1900 - NECESARIO SOLO LA PRIMERA VEZ
const char atc_1[] = "AT+CMGF=1";               //INICIA SMS MODOS TEXTO
//const char atc_3[] = "AT+CSCA=+5294100001410,145";       //CAMBIA NUMERO SMS - NECESARIO SOLO LA PRIMERA VEZ
const char atc_4[] = "AT+CNMI=0,0,0,0,0";           //INHABILITA EL AVISO SMS
const char atc_5[] = "AT+CMGS=9611768988";     //NUMERO AL QUE MANDA EL MENSAJE

// MENSAJES SMS A ENVIAR
const char sms_mj1[] = "MENSAJE DE PRUEBA";
const char sms_mj2[] = "ALARMA DE GAS";
const char sms_mj3[] = "ALARMA DE HUMO";
const char sms_mj4[] = "TOCANDO TIMBRE";
const char sms_mj5[] = "VENTANA ROTA";

// MENSAJES DEL LCD
const LCD_MESSAGE_LENGTH = 16;
const char lcd1[] = "Inicializando...";
const char lcd2[] = "Encendiendo GSM";
const char lcd3[] = "* -Continuar";
const char lcd4[] = "GSM Preparado";
const char lcd5[] = "# -MS1 0 -REES";
const char lcd6[] = "Enviando...";
const char lcd7[] = "Reestablecido";

char msg[LCD_MESSAGE_LENGTH+1];
short int i=0;

//DEFINICION DE VARIABLES MANEJO XBEE
unsigned char buff[14];
volatile unsigned char contador = 0x00;
volatile unsigned char dir_xbee = 0x00;
volatile unsigned char xbee_flag = 0;

//RUTINA DE LLENADO DE BUFFER DE RECEPCION
void Receptor(){
buff[contador] = UART1_Read();
if(buff[0] = 0x7E)
   contador++;
}

//RUTINA DE RESET DEL BUFFER DE RECEPCION
void Buff_Cleaner(){
unsigned char i;
for(i=0; i <= 14; i++){
   buff[i] = 0;
}
}

//RUTINA DE IDENTIFICACION DE BUFFER LLENO
unsigned short Buffer_Lleno(){
if(contador >= 14){
   contador = 0x00;
   return(1);
   }
else
   return(0);
}

//RUTINA DE IDENTIFICACION DE DIRECCION DE MODULO
unsigned char Check_Direccion(){
switch(buff[4]){
   case 0x11: return(0x01); break;
   case 0x22: return(0x02); break;
   case 0x33: return(0x03); break;
   case 0x44: return(0x04); break;
   default: return(0); break;
   }
}

// copy const to ram string
char * CopyConst2Ram(char * dest, const char * src){
char * d ;
 d = dest;
 for(;*dest++ = *src++;)
  ;
 return d;
}

void interrupt(){
if(RX_Flag){
   Receptor();
   if(Buffer_Lleno()){
      //DESHABILITAR INTERUPCION RX DE UART
      PIE1.RCIE   = 0;
      INTCON.PEIE = 0;
      INTCON.GIE  = 0;
      dir_xbee = Check_Direccion();
      xbee_flag = 1;
      Buff_Cleaner();
      }
   }
}

// send ATC command
void send_atc(const char *s)
{
// send command string
   while(*s) {
      UART1_Write(*s++);
   }
// terminate command with CR
   UART1_Write(0x0D);
}

// send SMS command
void send_sms(const char *s)
{
// send command string
   while(*s) {
      UART1_Write(*s++);
   }
// terminate command with CTRL Z
   UART1_Write(0x1A);
}

// pause
void wait() {
  Delay_ms(1000);
}
////////////////////////////////////////////////////////////////////////////////
void main() {

// all pins as digital I/Os
  ANSEL  = 0;
  ANSELH = 0;

// set PORTD inputs
  TRISD0_bit = 1;                                // place a call input
  TRISD1_bit = 1;                                // hang up input
  TRISD2_bit = 1;                                // answer a call input
  TRISD3_bit = 1;                                // continue with PIC program
  TRISD4_bit = 0;

  TRISE.B0 = 0;

// set RTS pin to zero, we will use only RX i TX
//  TRISC.F2 = 0;
//  PORTC.F2 = 0;

// disable uart rx interrupt
  PIE1.RCIE   = 0;
  INTCON.PEIE = 0;
  INTCON.GIE  = 0;

// setup lcd module
  Lcd_Init();
  Lcd_Cmd(_LCD_CURSOR_OFF);

// LCD_Out: power-up gsm
  LCD_Out(1,1,CopyConst2Ram(msg,lcd2));
  LCD_Out(2,1,CopyConst2Ram(msg,lcd3));

  UART1_init(9600);                             // initialize USART module
  Delay_ms(3000);
  //Encender modulo GSM
  PORTE.B0 = 1;
  Delay_ms(2000);
  PORTE.B0 = 0;
// wait for start
  PORTD.B4 = 1;
  while(!PORTD.F3)
   ;

  Lcd_Cmd(_LCD_CLEAR);
// LCD_Out: Initializing...
  LCD_Out(1,1,CopyConst2Ram(msg,lcd1));

  Delay_ms(5000);                                // wait for the GSM module to initialize it self

// negotiate baud rate
    do{
    send_atc("AT");                              // send "AT" string until gsm sets up its baud rade
    Delay_ms(100);
    i++;}
    while(i<=40);
    i=0;

// disable command echo
  send_atc(atc1);
  delay_ms(300);
// set full functionality
  send_atc(atc2);
  delay_ms(300);
  send_atc(atc_1);
  delay_ms(300);
  send_atc(atc_4);
  delay_ms(300);

  while(1) {
// LCD_Out: Ready! RB0-Send
    LCD_Out(1,1,CopyConst2Ram(msg,lcd4));
// LCD_Out: RB1-MS1 RB2-MS2
    LCD_Out(2,1,CopyConst2Ram(msg,lcd5));

    if(Button(&PORTD, 1, 10, 1)) {
// LCD_Out: Sending...
      Lcd_Cmd(_LCD_CLEAR);
      LCD_Out(1,1,CopyConst2Ram(msg,lcd6));

// mobile originated call to specified number
   send_atc(atc_5);
   delay_ms(300);
   send_sms(sms_mj1);
    delay_ms(300);
          Lcd_Cmd(_LCD_CLEAR);
    }

    if(Button(&PORTD, 2, 10, 1)) {
    while(UART1_Data_Ready()){
       UART1_Read();
       }
    Lcd_Cmd(_LCD_CLEAR);
    LCD_Out(1,1,CopyConst2Ram(msg,lcd7));
    Delay_ms(1000);
    // enable uart rx interrupt
    PIE1.RCIE   = 1;
    INTCON.PEIE = 1;
    INTCON.GIE  = 1;
    }

  if(xbee_flag){
     if(dir_xbee == 1){
        Lcd_Cmd(_LCD_CLEAR);
        LCD_Out(1,1,CopyConst2Ram(msg,sms_mj5));
        Delay_ms(1000);
        send_atc(atc_5);
        delay_ms(300);
        send_sms(sms_mj5);
        delay_ms(300);
        xbee_flag = 0;
        }
     if(dir_xbee == 2){
        Lcd_Cmd(_LCD_CLEAR);
        LCD_Out(1,1,CopyConst2Ram(msg,sms_mj4));
        Delay_ms(1000);
        send_atc(atc_5);
        delay_ms(300);
        send_sms(sms_mj4);
        delay_ms(300);
        xbee_flag = 0;
        }
     if(dir_xbee == 3){
        Lcd_Cmd(_LCD_CLEAR);
        LCD_Out(1,1,CopyConst2Ram(msg,sms_mj3));
        Delay_ms(1000);
        send_atc(atc_5);
        delay_ms(300);
        send_sms(sms_mj3);
        delay_ms(300);
        xbee_flag = 0;
        }
     if(dir_xbee == 4){
        Lcd_Cmd(_LCD_CLEAR);
        LCD_Out(1,1,CopyConst2Ram(msg,sms_mj2));
        Delay_ms(1000);
        send_atc(atc_5);
        delay_ms(300);
        send_sms(sms_mj2);
        delay_ms(300);
        xbee_flag = 0;
        }
     }
  }
}
